/*
 * db_my_client.c
 *
 *  Created at: 27.08.2009
 *      Author: Chebotarev Roman
 */


#include "config.h" 

#include "common_includes.h"
#include "db2dhcp_types.h"
#include "log.h"
#include "dhcp_queue.h"

#ifdef _WIN32
#include <mysql.h>
#else
#include CONFIG_MYSQL_H
#endif

extern inline void free_query_result(query_result_t * result);
extern int server_shutdown_flag;

void * connect_to_db_mysql(const char * host, const char * user,
		const char * passwd,	const char * db_name, uint16_t port)
{
	MYSQL * my = mysql_init(NULL);
	my->reconnect = 1;

	while(1)
	{
		if(!my)
		{
			log_wr(CLOG, "Can't create MySQL struct.", -1);
			return NULL;
		}

		log_wr(DLOG, "Connecting to MySQL DB '%s' on server '%s:%d' as user '%s'.",
				db_name, host, port, user);

		if(mysql_real_connect(my, host, user, passwd, db_name, port, 0, 0))
		{
			log_wr(NLOG, "Connected to MySQL server.");
			return my;
		}
		log_wr(ELOG, "Failed connect to MySQL database: %s. Sleeping %d seconds before reconnect.",
			mysql_error(my), DB_RECONNECT_TIME);
		sleep(DB_RECONNECT_TIME);

		if(server_shutdown_flag)
		{
			log_wr(NLOG, "Shuting down MySQL thread.");
			return NULL;
		}
	}

	return NULL;
}

void * query_mysql(void * dbh, const char *sql_st, int st_len)
{
	MYSQL * my = dbh;

	log_wr(DLOG, "Executing SQL statement \"%s\"", sql_st);

	if(mysql_real_query(my, sql_st, st_len))
	{
		log_wr(ELOG, "Error executing SQL statement: \"%s\"", mysql_error(my));
		return NULL;
	}

	MYSQL_RES * result = mysql_store_result(my);
	log_wr(DLOG, "Fetched "I64U" rows, %u fields from DB.", result->row_count, result->field_count);
	if(result->field_count < VALUE_INDEX + 1)
	{
		log_wr(ELOG, "Too few fields in fetched result. Check you request!");
		return NULL;
	}


	query_result_t *out_result = calloc(1, sizeof(query_result_t));
	if(!out_result)
	{
		log_wr(CLOG, "Can't allocate memory for store query result!");
		mysql_free_result(result);
		return NULL;
	}

	out_result->items = malloc(sizeof(result_item_t) * result->row_count);
	if(!out_result->items)
	{
		log_wr(CLOG, "Can't allocate memory for store query result nodes!");
		mysql_free_result(result);
		free(out_result);
		return NULL;
	}

	int i;
	unsigned long *lengths;
	MYSQL_ROW row;
	for(i = 0; (row = mysql_fetch_row(result)); ++i)
	{
		lengths = mysql_fetch_lengths(result);
		/* Fields order: code, value */
		out_result->items[i].len = lengths[VALUE_INDEX];
		out_result->items[i].code = atoi(row[CODE_INDEX]);
		out_result->items[i].type = atoi(row[TYPE_INDEX]);
		out_result->items[i].data = malloc(lengths[VALUE_INDEX] +
		                                  (out_result->items[i].type == BINARY ? 0 : 1));

		if(!out_result->items[i].data)
		{
			log_wr(CLOG, "Can't allocate memory for store data (code %d, length %d).",
					row[CODE_INDEX], lengths[VALUE_INDEX]);
			out_result->count = i;
			free_query_result(out_result);
			mysql_free_result(result);
			return NULL;
		}
		memcpy(out_result->items[i].data, row[VALUE_INDEX], lengths[VALUE_INDEX]);
		if(out_result->items[i].type != BINARY)	/* Not binary data must be zero-terminated */
			((char*)(out_result->items[i].data))[out_result->items[i].len] = '\0';
	}

	out_result->count = i;
	mysql_free_result(result);

	return out_result;
}

void disconnect_from_mysql(void * dbm)
{
	log_wr(NLOG, "Disconnecting from MySQL database.");
	mysql_close((MYSQL*) dbm);

	return; 
}
